@@ -59,8 +59,10 @@ void CloudStorageAuthenticate::uploadFinished()
 			nw->hideNotification();
 		*/
 		myLastError.clear();
-	} else if (cloudAuthReply == QLatin1String("[VERIFY]")) {
+	} else if (cloudAuthReply == QLatin1String("[VERIFY]") ||
+		   cloudAuthReply == QLatin1String("Invalid PIN")) {
 		csSettings.setVerificationStatus(CS_NEED_TO_VERIFY);
+		report_error(qPrintable(tr("Cloud account verification required, enter PIN in preferences")));
 	} else if (cloudAuthReply == QLatin1String("[PASSWDCHANGED]")) {
 		free(prefs.cloud_storage_password);
 		prefs.cloud_storage_password = prefs.cloud_storage_newpassword;
