@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "core/gpslocation.h"
 #include "qt-models/gpslistmodel.h"
 #include "core/pref.h"
@@ -45,11 +46,24 @@ GpsLocation *GpsLocation::instance()
 	return m_Instance;
 }
 
+bool GpsLocation::hasInstance()
+{
+	return m_Instance != NULL;
+}
+
 GpsLocation::~GpsLocation()
 {
 	m_Instance = NULL;
 }
 
+void GpsLocation::setGpsTimeThreshold(int seconds)
+{
+	if (m_GpsSource) {
+		m_GpsSource->setUpdateInterval(seconds * 1000);
+		status(QString("Set GPS service update interval to %1").arg(m_GpsSource->updateInterval()));
+	}
+}
+
 QGeoPositionInfoSource *GpsLocation::getGpsSource()
 {
 	if (haveSource == NOGPS)
@@ -80,7 +94,7 @@ QGeoPositionInfoSource *GpsLocation::getGpsSource()
 			connect(m_GpsSource, SIGNAL(positionUpdated(QGeoPositionInfo)), this, SLOT(newPosition(QGeoPositionInfo)));
 			connect(m_GpsSource, SIGNAL(updateTimeout()), this, SLOT(updateTimeout()));
 			connect(m_GpsSource, SIGNAL(error(QGeoPositionInfoSource::Error)), this, SLOT(positionSourceError(QGeoPositionInfoSource::Error)));
-			m_GpsSource->setUpdateInterval(5 * 60 * 1000); // 5 minutes so the device doesn't drain the battery
+			setGpsTimeThreshold(prefs.time_threshold);
 		} else {
 #ifdef SUBSURFACE_MOBILE
 			status("don't have GPS source");
@@ -157,8 +171,8 @@ void GpsLocation::newPosition(QGeoPositionInfo pos)
 		gpsTracker gt;
 		gt.when = pos.timestamp().toTime_t();
 		gt.when += gettimezoneoffset(gt.when);
-		gt.latitude.udeg = (int)(pos.coordinate().latitude() * 1000000);
-		gt.longitude.udeg = (int)(pos.coordinate().longitude() * 1000000);
+		gt.latitude.udeg = lrint(pos.coordinate().latitude() * 1000000);
+		gt.longitude.udeg = lrint(pos.coordinate().longitude() * 1000000);
 		addFixToStorage(gt);
 	}
 }
@@ -261,7 +275,7 @@ bool GpsLocation::applyLocations()
 					qDebug() << "processing gpsFix @" << get_dive_date_string(gpsTable[j].when) <<
 						    "which is withing six hours of dive from" <<
 						    get_dive_date_string(d->when) << "until" <<
-						    get_dive_date_string(d->when + d->duration.seconds);
+						    get_dive_date_string(dive_endtime(d));
 				/*
 				 * If position is fixed during dive. This is the good one.
 				 * Asign and mark position, and end gps_location loop
@@ -296,7 +310,7 @@ bool GpsLocation::applyLocations()
 							if (verbose)
 								qDebug() << "which is closer to the start of the dive, do continue with that";
 							continue;
-						} else if (gpsTable[j].when > d->when + d->duration.seconds) {
+						} else if (gpsTable[j].when > dive_endtime(d)) {
 							if (verbose)
 								qDebug() << "which is even later after the end of the dive, so pick the previous one";
 							copy_gps_location(gpsTable[j], d);
@@ -305,7 +319,7 @@ bool GpsLocation::applyLocations()
 							break;
 						} else {
 							/* ok, gpsFix is before, nextgpsFix is after */
-							if (d->when - gpsTable[j].when <= gpsTable[j+1].when - (d->when + d->duration.seconds)) {
+							if (d->when - gpsTable[j].when <= gpsTable[j+1].when - dive_endtime(d)) {
 								if (verbose)
 									qDebug() << "pick the one before as it's closer to the start";
 								copy_gps_location(gpsTable[j], d);
@@ -337,7 +351,7 @@ bool GpsLocation::applyLocations()
 				/* If position is out of SAME_GROUP range and in the future, mark position for
 				 * next dive iteration and end the gps_location loop
 				 */
-				if (gpsTable[j].when >= d->when + d->duration.seconds + SAME_GROUP) {
+				if (gpsTable[j].when >= dive_endtime(d) + SAME_GROUP) {
 					last = j;
 					break;
 				}
@@ -612,8 +626,8 @@ void GpsLocation::downloadFromServer()
 
 				struct gpsTracker gt;
 				gt.when = timestamp.toMSecsSinceEpoch() / 1000;
-				gt.latitude.udeg = (int)(latitude.toDouble() * 1000000);
-				gt.longitude.udeg = (int)(longitude.toDouble() * 1000000);
+				gt.latitude.udeg = lrint(latitude.toDouble() * 1000000);
+				gt.longitude.udeg = lrint(longitude.toDouble() * 1000000);
 				gt.name = name;
 				// add this GPS fix to the QMap and the settings (remove existing fix at the same timestamp first)
 				if (m_trackers.keys().contains(gt.when)) {
