@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifdef __clang__
 // Clang has a bug on zero-initialization of C structs.
 #pragma clang diagnostic ignored "-Wmissing-field-initializers"
@@ -15,7 +16,13 @@
 
 #include <libdivecomputer/version.h>
 #include "libdivecomputer.h"
+#include "core/version.h"
 
+#if !defined(SSRF_LIBDC_VERSION) || SSRF_LIBDC_VERSION < 2
+#pragma message "Subsurface requires a reasonably current version of the Subsurface-branch"
+#pragma message "of libdivecomputer (at least version 2 of our API)."
+#pragma message "Please get it from http://github.com/Subsurface-divelog/libdc Subsurface-branch"
+#endif
 //
 // If we have an old libdivecomputer, it doesn't
 // have the new DC_TANKINFO bits, but just volume
@@ -31,12 +38,23 @@
 char *dumpfile_name;
 char *logfile_name;
 const char *progress_bar_text = "";
+void (*progress_callback)(const char *text) = NULL;
 double progress_bar_fraction = 0.0;
 
 static int stoptime, stopdepth, ndl, po2, cns;
 static bool in_deco, first_temp_is_air;
 static int current_gas_index;
 
+/* logging bits from libdivecomputer */
+#ifndef __ANDROID__
+#define INFO(context, fmt, ...)	fprintf(stderr, "INFO: " fmt "\n", ##__VA_ARGS__)
+#define ERROR(context, fmt, ...)	fprintf(stderr, "ERROR: " fmt "\n", ##__VA_ARGS__)
+#else
+#include <android/log.h>
+#define INFO(context, fmt, ...)	__android_log_print(ANDROID_LOG_DEBUG, __FILE__, "INFO: " fmt "\n", ##__VA_ARGS__)
+#define ERROR(context, fmt, ...)	__android_log_print(ANDROID_LOG_DEBUG, __FILE__, "ERROR: " fmt "\n", ##__VA_ARGS__)
+#endif
+
 /*
  * Directly taken from libdivecomputer's examples/common.c to improve
  * the error messages resulting from libdc's return codes
@@ -336,15 +354,7 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 		sample->depth.mm = lrint(value.depth * 1000);
 		break;
 	case DC_SAMPLE_PRESSURE:
-		/* Do we already have a pressure reading? */
-		if (sample->cylinderpressure.mbar) {
-			/* Do we prefer the one we already have? */
-			/* If so, just ignore the new one */
-			if (sample->sensor == current_gas_index)
-				break;
-		}
-		sample->sensor = value.pressure.tank;
-		sample->cylinderpressure.mbar = lrint(value.pressure.value * 1000);
+		add_sample_pressure(sample, value.pressure.tank, lrint(value.pressure.value * 1000));
 		break;
 	case DC_SAMPLE_GASMIX:
 		handle_gasmix(dc, sample, value.gasmix);
@@ -423,6 +433,8 @@ static void dev_info(device_data_t *devdata, const char *fmt, ...)
 	vsnprintf(buffer, sizeof(buffer), fmt, ap);
 	va_end(ap);
 	progress_bar_text = buffer;
+	if (progress_callback)
+		(*progress_callback)(buffer);
 }
 
 static int import_dive_number = 0;
@@ -480,7 +492,7 @@ static int find_dive(struct divecomputer *match)
 {
 	int i;
 
-	for (i = 0; i < dive_table.preexisting; i++) {
+	for (i = dive_table.preexisting - 1; i >= 0; i--) {
 		struct dive *old = dive_table.dives[i];
 
 		if (match_one_dive(match, old))
@@ -584,7 +596,7 @@ static void parse_string_field(struct dive *dive, dc_field_string_t *str)
 }
 #endif
 
-static dc_status_t libdc_header_parser(dc_parser_t *parser, struct device_data_t *devdata, struct dive *dive)
+static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devdata, struct dive *dive)
 {
 	dc_status_t rc = 0;
 	dc_datetime_t dt = { 0 };
@@ -788,8 +800,11 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	}
 
 	/* If we already saw this dive, abort. */
-	if (!devdata->force_download && find_dive(&dive->dc))
+	if (!devdata->force_download && find_dive(&dive->dc)) {
+		const char *date_string = get_dive_date_c_string(dive->when);
+		dev_info(devdata, translate("gettextFromC", "Already downloaded dive at %s"), date_string);
 		goto error_exit;
+	}
 
 	dc_parser_destroy(parser);
 
@@ -929,10 +944,19 @@ static void event_cb(dc_device_t *device, dc_event_type_t event, const void *dat
 		progress_bar_fraction = (double)progress->current / (double)progress->maximum;
 		break;
 	case DC_EVENT_DEVINFO:
-		dev_info(devdata, translate("gettextFromC", "model=%u (0x%08x), firmware=%u (0x%08x), serial=%u (0x%08x)"),
-			 devinfo->model, devinfo->model,
-			 devinfo->firmware, devinfo->firmware,
-			 devinfo->serial, devinfo->serial);
+		if (dc_descriptor_get_model(devdata->descriptor) != devinfo->model) {
+			fprintf(stderr, "EVENT_DEVINFO gave us the correct detected product (model %d instead of %d)\n",
+				devinfo->model, dc_descriptor_get_model(devdata->descriptor));
+			dc_descriptor_t *better_descriptor = get_descriptor(dc_descriptor_get_type(devdata->descriptor), devinfo->model);
+			if (better_descriptor != NULL) {
+				devdata->descriptor = better_descriptor;
+				devdata->product = dc_descriptor_get_product(better_descriptor);
+				devdata->vendor = dc_descriptor_get_vendor(better_descriptor);
+				devdata->model = str_printf("%s %s", devdata->vendor, devdata->product);
+			}
+		}
+		dev_info(devdata, translate("gettextFromC", "model=%s firmware=%u serial=%u"),
+			 devdata->product, devinfo->firmware, devinfo->serial);
 		if (devdata->libdc_logfile) {
 			fprintf(devdata->libdc_logfile, "Event: model=%u (0x%08x), firmware=%u (0x%08x), serial=%u (0x%08x)\n",
 				devinfo->model, devinfo->model,
@@ -1063,18 +1087,23 @@ const char *do_libdivecomputer_import(device_data_t *data)
 	if (fp) {
 		dc_context_set_loglevel(data->context, DC_LOGLEVEL_ALL);
 		dc_context_set_logfunc(data->context, logfunc, fp);
+		fprintf(data->libdc_logfile, "Subsurface: v%s, ", subsurface_git_version());
+		fprintf(data->libdc_logfile, "built with libdivecomputer v%s\n", dc_version(NULL));
 	}
 
 	err = translate("gettextFromC", "Unable to open %s %s (%s)");
 
-#if defined(SSRF_CUSTOM_SERIAL)
+#if defined(SSRF_CUSTOM_IO)
 	if (data->bluetooth_mode) {
-#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_SERIAL)
-		rc = dc_context_set_custom_serial(data->context, get_qt_serial_ops());
+#if defined(BT_SUPPORT) && defined(SSRF_CUSTOM_IO)
+		rc = dc_context_set_custom_io(data->context, get_qt_serial_ops(), data);
 #endif
 #ifdef SERIAL_FTDI
 	} else if (!strcmp(data->devname, "ftdi")) {
-		rc = dc_context_set_custom_serial(data->context, &serial_ftdi_ops);
+		rc = dc_context_set_custom_io(data->context, &serial_ftdi_ops, data);
+		INFO(0, "setting up ftdi ops");
+#else
+		INFO(0, "FTDI disabled");
 #endif
 	}
 
@@ -1085,9 +1114,9 @@ const char *do_libdivecomputer_import(device_data_t *data)
 	{
 #endif
 		rc = dc_device_open(&data->device, data->context, data->descriptor, data->devname);
-
+		INFO(0, "dc_deveice_open error value of %d", rc);
 		if (rc != DC_STATUS_SUCCESS && subsurface_access(data->devname, R_OK | W_OK) != 0)
-			err = translate("gettextFromC", "Insufficient privileges to open the device %s %s (%s)");
+			err = translate("gettextFromC", "Error opening the device %s %s (%s).\nIn most cases, in order to debug this issue, a libdivecomputer logfile will be useful.\nYou can create this logfile by selecting the corresponding checkbox in the download dialog.");
 	}
 
 	if (rc == DC_STATUS_SUCCESS) {
@@ -1095,6 +1124,8 @@ const char *do_libdivecomputer_import(device_data_t *data)
 		/* TODO: Show the logfile to the user on error. */
 		dc_device_close(data->device);
 		data->device = NULL;
+		if (!downloadTable.nr)
+			dev_info(data, translate("gettextFromC", "No new dives downloaded from dive computer"));
 	}
 
 	dc_context_free(data->context);
