@@ -34,7 +34,6 @@ int decostoplevels_imperial[] = { 0, 3048, 6096, 9144, 12192, 15240, 18288, 2133
 				325120, 345440, 365760, 386080 };
 
 double plangflow, plangfhigh;
-int bottom_time = 0;
 
 extern double regressiona();
 extern double regressionb();
@@ -653,13 +652,13 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	int bottom_gi;
 	int bottom_stopidx;
 	bool is_final_plan = true;
-	int deco_time;
+	int bottom_time;
 	int previous_deco_time;
 	struct deco_state *bottom_cache = NULL;
 	struct sample *sample;
 	int po2;
 	int transitiontime, gi;
-	int current_cylinder, stopcylinder;
+	int current_cylinder, stop_cylinder;
 	int stopidx;
 	int depth;
 	struct gaschanges *gaschanges = NULL;
@@ -674,9 +673,10 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	int last_ascend_rate;
 	int best_first_ascend_cylinder;
 	struct gasmix gas, bottom_gas;
-	int o2time = 0;
-	int breaktime = -1;
+	bool o2break_next = false;
+	bool o2break_done = false;
 	int break_cylinder = -1, breakfrom_cylinder = 0;
+	bool last_segment_min_switch = false;
 	int error = 0;
 	bool decodive = false;
 	int first_stop_depth = 0;
@@ -713,6 +713,9 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	/* Let's start at the last 'sample', i.e. the last manually entered waypoint. */
 	sample = &dive->dc.sample[dive->dc.samples - 1];
 
+	/* Keep time during the ascend */
+	bottom_time = clock = previous_point_time = dive->dc.sample[dive->dc.samples - 1].time.seconds;
+
 	current_cylinder = get_cylinderid_at_time(dive, &dive->dc, sample->time);
 	gas = dive->cylinder[current_cylinder].gasmix;
 
@@ -754,8 +757,6 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	stoplevels = sort_stops(decostoplevels, stopidx + 1, gaschanges, gaschangenr);
 	stopidx += gaschangenr;
 
-	/* Keep time during the ascend */
-	bottom_time = clock = previous_point_time = dive->dc.sample[dive->dc.samples - 1].time.seconds;
 	gi = gaschangenr - 1;
 
 	/* Set tissue tolerance and initial vpmb gradient at start of ascent phase */
@@ -830,7 +831,7 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	// VPM-B or Buehlmann Deco
 	tissue_at_end(dive, cached_datap);
 	previous_deco_time = 100000000;
-	deco_time = 10000000;
+	deco_state->deco_time = 10000000;
 	cache_deco_state(&bottom_cache);  // Lets us make several iterations
 	bottom_depth = depth;
 	bottom_gi = gi;
@@ -840,11 +841,11 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 	//CVA
 	do {
 		decostopcounter = 0;
-		is_final_plan = (decoMode() == BUEHLMANN) || (previous_deco_time - deco_time < 10);  // CVA time converges
-		if (deco_time != 10000000)
-			vpmb_next_gradient(deco_time, diveplan->surface_pressure / 1000.0);
+		is_final_plan = (decoMode() == BUEHLMANN) || (previous_deco_time - deco_state->deco_time < 10);  // CVA time converges
+		if (deco_state->deco_time != 10000000)
+			vpmb_next_gradient(deco_state->deco_time, diveplan->surface_pressure / 1000.0);
 
-		previous_deco_time = deco_time;
+		previous_deco_time = deco_state->deco_time;
 		restore_deco_state(bottom_cache, true);
 
 		depth = bottom_depth;
@@ -855,8 +856,6 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 		decodive = false;
 		first_stop_depth = 0;
 		stopidx = bottom_stopidx;
-		breaktime = -1;
-		o2time = 0;
 		deco_state->first_ceiling_pressure.mbar = depth_to_mbar(deco_allowed_depth(tissue_tolerance_calc(dive,
 												     depth_to_bar(depth, dive)),
 									    diveplan->surface_pressure / 1000.0,
@@ -896,6 +895,7 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 				add_segment(depth_to_bar(depth, dive),
 								&dive->cylinder[current_cylinder].gasmix,
 								TIMESTEP, po2, dive, prefs.decosac);
+				last_segment_min_switch = false;
 				clock += TIMESTEP;
 				depth -= deltad;
 				/* Print VPM-Gradient as gradient factor, this has to be done from within deco.c */
@@ -929,12 +929,13 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 						printf("switch to gas %d (%d/%d) @ %5.2lfm\n", gaschanges[gi].gasidx,
 							(get_o2(&gas) + 5) / 10, (get_he(&gas) + 5) / 10, gaschanges[gi].depth / 1000.0);
 #endif
-						/* Stop for the minimum duration to switch gas */
-						if (!(prefs.doo2breaks && get_o2(&dive->cylinder[current_cylinder].gasmix) == 1000)) {
+						/* Stop for the minimum duration to switch gas unless we switch to o2 */
+						if (!last_segment_min_switch && get_o2(&dive->cylinder[current_cylinder].gasmix) != 1000) {
 							add_segment(depth_to_bar(depth, dive),
 								&dive->cylinder[current_cylinder].gasmix,
 								prefs.min_switch_duration, po2, dive, prefs.decosac);
 							clock += prefs.min_switch_duration;
+							last_segment_min_switch = true;
 						}
 					} else {
 						/* The user has selected the option to switch gas only at required stops.
@@ -982,12 +983,13 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 					printf("switch to gas %d (%d/%d) @ %5.2lfm\n", gaschanges[gi + 1].gasidx,
 						(get_o2(&gas) + 5) / 10, (get_he(&gas) + 5) / 10, gaschanges[gi + 1].depth / 1000.0);
 #endif
-					/* Stop for the minimum duration to switch gas */
-					if (!(prefs.doo2breaks && get_o2(&dive->cylinder[current_cylinder].gasmix) == 1000)) {
+					/* Stop for the minimum duration to switch gas unless we switch to o2 */
+					if (!last_segment_min_switch && get_o2(&dive->cylinder[current_cylinder].gasmix) != 1000) {
 						add_segment(depth_to_bar(depth, dive),
 							&dive->cylinder[current_cylinder].gasmix,
 							prefs.min_switch_duration, po2, dive, prefs.decosac);
 						clock += prefs.min_switch_duration;
+						last_segment_min_switch = true;
 					}
 					pendinggaschange = false;
 				}
@@ -1001,8 +1003,8 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 				}
 
 				o2breaking = false;
-				stopcylinder = current_cylinder;
-				if (prefs.doo2breaks) {
+				stop_cylinder = current_cylinder;
+				if (prefs.doo2breaks && prefs.last_stop) {
 					/* The backgas breaks option limits time on oxygen to 12 minutes, followed by 6 minutes on
 					 * backgas.  This could be customized if there were demand.
 					 */
@@ -1017,7 +1019,8 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 							laststoptime = 12 * 60;
 							new_clock = clock + laststoptime;
 							o2breaking = true;
-							breaktime = 0;
+							o2break_next = true;
+							o2break_done = true;
 							breakfrom_cylinder = current_cylinder;
 							if (is_final_plan)
 								plan_add_segment(diveplan, laststoptime, depth, current_cylinder, po2, false);
@@ -1025,25 +1028,23 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 							current_cylinder = break_cylinder;
 							gas = dive->cylinder[current_cylinder].gasmix;
 						}
-					} else {
-						if (breaktime >= 0) {
-							if (laststoptime >= 6 * 60) {
-								laststoptime = 6 * 60;
-								new_clock = clock + laststoptime;
-								o2breaking  = true;
-								o2time = 0;
-								if (is_final_plan)
-									plan_add_segment(diveplan, laststoptime, depth, current_cylinder, po2, false);
-								previous_point_time = clock + laststoptime;
-								current_cylinder = breakfrom_cylinder;
-								gas = dive->cylinder[current_cylinder].gasmix;
-								breaktime = -1;
-							}
+					} else if (o2break_next) {
+						if (laststoptime >= 6 * 60) {
+							laststoptime = 6 * 60;
+							new_clock = clock + laststoptime;
+							o2breaking  = true;
+							o2break_next = false;
+							if (is_final_plan)
+								plan_add_segment(diveplan, laststoptime, depth, current_cylinder, po2, false);
+							previous_point_time = clock + laststoptime;
+							current_cylinder = breakfrom_cylinder;
+							gas = dive->cylinder[current_cylinder].gasmix;
 						}
 					}
 				}
-				add_segment(depth_to_bar(depth, dive), &dive->cylinder[stopcylinder].gasmix,
+				add_segment(depth_to_bar(depth, dive), &dive->cylinder[stop_cylinder].gasmix,
 					    laststoptime, po2, dive, prefs.decosac);
+				last_segment_min_switch = false;
 				decostoptable[decostopcounter].depth = depth;
 				decostoptable[decostopcounter].time = laststoptime;
 				++decostopcounter;
@@ -1060,8 +1061,11 @@ bool plan(struct diveplan *diveplan, struct dive *dive, int timestep, struct dec
 				stopping = false;
 			}
 		}
-
-		deco_time = clock - bottom_time;
+		/* When calculating deco_time, we should pretend the final ascent rate is always the same,
+		 * otherwise odd things can happen, such as CVA causing the final ascent to start *later*
+		 * if the ascent rate is slower, which is completely nonsensical.
+		 * Assume final ascent takes 20s, which is the time taken to ascend at 9m/min from 3m */
+		deco_state->deco_time = clock - bottom_time - stoplevels[stopidx + 1] / last_ascend_rate + 20;
 	} while (!is_final_plan);
 	decostoptable[decostopcounter].depth = 0;
 
