@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #include <errno.h>
 
 #include <QtBluetooth/QBluetoothAddress>
@@ -7,8 +8,9 @@
 #include <QDebug>
 
 #include <libdivecomputer/version.h>
+#include <libdivecomputer/context.h>
 
-#if defined(SSRF_CUSTOM_SERIAL)
+#if defined(SSRF_CUSTOM_IO)
 
 #if defined(Q_OS_WIN)
 	#include <winsock2.h>
@@ -16,7 +18,18 @@
 	#include <ws2bth.h>
 #endif
 
-#include <libdivecomputer/custom_serial.h>
+#include <libdivecomputer/custom_io.h>
+
+#ifdef BLE_SUPPORT
+# include "qt-ble.h"
+#endif
+
+QList<QBluetoothUuid> registeredUuids;
+
+void addBtUuid(QBluetoothUuid uuid)
+{
+	registeredUuids << uuid;
+}
 
 extern "C" {
 typedef struct qt_serial_t {
@@ -31,8 +44,198 @@ typedef struct qt_serial_t {
 	long timeout;
 } qt_serial_t;
 
-static dc_status_t qt_serial_open(void **userdata, const char* devaddr)
+#ifdef BLE_SUPPORT
+
+static dc_status_t ble_serial_open(dc_custom_io_t *io, dc_context_t *, const char* devaddr);
+static dc_status_t ble_serial_close(dc_custom_io_t *io);
+static dc_status_t ble_serial_read(dc_custom_io_t *io, void* data, size_t size, size_t *actual);
+static dc_status_t ble_serial_write(dc_custom_io_t *io, const void* data, size_t size, size_t *actual);
+static dc_status_t ble_serial_purge(dc_custom_io_t *io, dc_direction_t queue);
+static dc_status_t ble_serial_get_available(dc_custom_io_t *io, size_t *available);
+static dc_status_t ble_serial_set_timeout(dc_custom_io_t *io, long timeout);
+
+static dc_custom_io_t ble_serial_ops = {
+	.userdata = NULL,
+	.user_device = NULL,
+
+	.serial_open = ble_serial_open,
+	.serial_close = ble_serial_close,
+	.serial_read = ble_serial_read,
+	.serial_write = ble_serial_write,
+	.serial_purge = ble_serial_purge,
+	.serial_get_available = ble_serial_get_available,
+	.serial_set_timeout = ble_serial_set_timeout,
+// These doesn't make sense over bluetooth
+// NULL means NOP
+	.serial_configure = NULL,
+	.serial_set_dtr = NULL,
+	.serial_set_rts = NULL,
+	.serial_set_halfduplex = NULL,
+	.serial_set_break = NULL,
+
+	.packet_size  = 20,
+	.packet_open  = qt_ble_open,
+	.packet_close = qt_ble_close,
+	.packet_read  = qt_ble_read,
+	.packet_write = qt_ble_write,
+};
+
+
+static dc_status_t ble_serial_open(dc_custom_io_t *io, dc_context_t *context, const char* devaddr)
 {
+	dc_context_set_custom_io(context, &ble_serial_ops, io->user_device);
+	return qt_ble_open(&ble_serial_ops, context, devaddr);
+}
+
+#define BT_BLE_BUFSIZE 4096
+static struct {
+	unsigned int out_bytes, in_bytes, in_pos;
+	unsigned char in[BT_BLE_BUFSIZE];
+	unsigned char out[BT_BLE_BUFSIZE];
+} buffer;
+
+static dc_status_t ble_serial_flush_write(void)
+{
+	int bytes = buffer.out_bytes;
+
+	if (!bytes)
+		return DC_STATUS_SUCCESS;
+	buffer.out_bytes = 0;
+	return ble_serial_ops.packet_write(&ble_serial_ops, buffer.out, bytes, NULL);
+}
+
+static dc_status_t ble_serial_flush_read(void)
+{
+	buffer.in_bytes = buffer.in_pos = 0;
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t ble_serial_close(dc_custom_io_t *io)
+{
+	ble_serial_flush_write();
+	io->userdata = NULL;
+	return qt_ble_close(&ble_serial_ops);
+}
+
+static dc_status_t ble_serial_read(dc_custom_io_t *io, void* data, size_t size, size_t *actual)
+{
+	Q_UNUSED(io)
+	size_t len;
+	size_t received = 0;
+
+	if (buffer.in_pos >= buffer.in_bytes) {
+		ble_serial_flush_write();
+	}
+
+	/* There is still unused/unread data in the input steam.
+	 * So preseve it at the start of a new read.
+	 */
+	if (buffer.in_pos > 0) {
+		len = buffer.in_bytes - buffer.in_pos;
+		memcpy(buffer.in, buffer.in + buffer.in_pos, len);
+		buffer.in_pos = 0;
+		buffer.in_bytes = len;
+	}
+
+	/* Read a long as requested in the size parameter */
+	while ((buffer.in_bytes - buffer.in_pos) < size) {
+		dc_status_t rc;
+
+		rc = ble_serial_ops.packet_read(&ble_serial_ops, buffer.in + buffer.in_bytes,
+						sizeof(buffer.in) - buffer.in_bytes, &received);
+		if (rc != DC_STATUS_SUCCESS)
+			return rc;
+		if (!received)
+			return DC_STATUS_IO;
+
+		buffer.in_bytes += received;
+	}
+
+	len = buffer.in_bytes - buffer.in_pos;
+	if (len > size)
+		len = size;
+
+	memcpy(data, buffer.in + buffer.in_pos, len);
+	buffer.in_pos += len;
+	if (actual)
+		*actual = len;
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t ble_serial_write(dc_custom_io_t *io, const void* data, size_t size, size_t *actual)
+{
+	Q_UNUSED(io)
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	size_t transferred = 0;
+
+	ble_serial_flush_read();
+
+	/*
+	 * Most writes to a connected DC are small, typically some command bytes to get
+	 * DC in download mode, or to set some parameter. All this just worked over BLE,
+	 * however, sending a full firmware update (on an OSTC device) failed, as the
+	 * underlying BLE interface can only handle small 20 byte BLE packets at once.
+	 *
+	 * So, send max ble->packet_size chuncks at once.
+	 */
+	while (size) {
+		size_t len = sizeof(buffer.out) - transferred;
+
+		if ((int)len > io->packet_size)
+			len = io->packet_size;
+		if (len > size)
+			len = size;
+		memcpy(buffer.out + buffer.out_bytes, data, len);
+		buffer.out_bytes += len;
+
+		if ((int)buffer.out_bytes <= io->packet_size || buffer.out_bytes == size) {
+			rc = ble_serial_flush_write();
+			if (rc != DC_STATUS_SUCCESS)
+				break;
+		}
+		transferred += len;
+		data = (const void *) (len + (const char *)data);
+		size -= len;
+	}
+	if (actual)
+		*actual = transferred;
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t ble_serial_purge(dc_custom_io_t *io, dc_direction_t queue)
+{
+	Q_UNUSED(io)
+	Q_UNUSED(queue)
+	/* Do we care? */
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t ble_serial_get_available(dc_custom_io_t *io, size_t *available)
+{
+	Q_UNUSED(io)
+	*available = buffer.in_bytes - buffer.in_pos;
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t ble_serial_set_timeout(dc_custom_io_t *io, long timeout)
+{
+	Q_UNUSED(io)
+	Q_UNUSED(timeout)
+	/* Do we care? */
+	return DC_STATUS_SUCCESS;
+}
+
+#endif
+
+
+
+static dc_status_t qt_serial_open(dc_custom_io_t *io, dc_context_t *context, const char* devaddr)
+{
+#ifdef BLE_SUPPORT
+	if (!strncmp(devaddr, "LE:", 3))
+		return ble_serial_open(io, context, devaddr);
+#endif
+
 	// Allocate memory.
 	qt_serial_t *serial_port = (qt_serial_t *) malloc (sizeof (qt_serial_t));
 	if (serial_port == NULL) {
@@ -132,7 +335,14 @@ static dc_status_t qt_serial_open(void **userdata, const char* devaddr)
 #elif defined(Q_OS_ANDROID) || (QT_VERSION >= 0x050500 && defined(Q_OS_MAC))
 	// Try to connect to the device using the uuid of the Serial Port Profile service
 	QBluetoothAddress remoteDeviceAddress(devaddr);
-	serial_port->socket->connectToService(remoteDeviceAddress, QBluetoothUuid(QBluetoothUuid::SerialPort));
+#if defined(Q_OS_ANDROID)
+	QBluetoothUuid uuid = QBluetoothUuid(QUuid("{00001101-0000-1000-8000-00805f9b34fb}"));
+	qDebug() << "connecting to Uuid" << uuid;
+	serial_port->socket->setPreferredSecurityFlags(QBluetooth::NoSecurity);
+	serial_port->socket->connectToService(remoteDeviceAddress, uuid, QIODevice::ReadWrite | QIODevice::Unbuffered);
+#else
+	serial_port->socket->connectToService(remoteDeviceAddress, 1, QIODevice::ReadWrite | QIODevice::Unbuffered);
+#endif
 	timer.start(msec);
 	loop.exec();
 
@@ -168,14 +378,14 @@ static dc_status_t qt_serial_open(void **userdata, const char* devaddr)
 		}
 	}
 #endif
-	*userdata = serial_port;
+	io->userdata = serial_port;
 
 	return DC_STATUS_SUCCESS;
 }
 
-static dc_status_t qt_serial_close(void **userdata)
+static dc_status_t qt_serial_close(dc_custom_io_t *io)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
 
 	if (device == NULL)
 		return DC_STATUS_SUCCESS;
@@ -196,14 +406,14 @@ static dc_status_t qt_serial_close(void **userdata)
 	free(device);
 #endif
 
-	*userdata = NULL;
+	io->userdata = NULL;
 
 	return DC_STATUS_SUCCESS;
 }
 
-static dc_status_t qt_serial_read(void **userdata, void* data, size_t size, size_t *actual)
+static dc_status_t qt_serial_read(dc_custom_io_t *io, void* data, size_t size, size_t *actual)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
 
 #if defined(Q_OS_WIN)
 	if (device == NULL)
@@ -262,9 +472,9 @@ static dc_status_t qt_serial_read(void **userdata, void* data, size_t size, size
 	return DC_STATUS_SUCCESS;
 }
 
-static dc_status_t qt_serial_write(void **userdata, const void* data, size_t size, size_t *actual)
+static dc_status_t qt_serial_write(dc_custom_io_t *io, const void* data, size_t size, size_t *actual)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
 
 #if defined(Q_OS_WIN)
 	if (device == NULL)
@@ -311,9 +521,10 @@ static dc_status_t qt_serial_write(void **userdata, const void* data, size_t siz
 	return DC_STATUS_SUCCESS;
 }
 
-static dc_status_t qt_serial_flush(void **userdata, dc_direction_t queue)
+static dc_status_t qt_serial_purge(dc_custom_io_t *io, dc_direction_t queue)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
+
 	(void)queue;
 	if (device == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -326,9 +537,10 @@ static dc_status_t qt_serial_flush(void **userdata, dc_direction_t queue)
 	return DC_STATUS_SUCCESS;
 }
 
-static dc_status_t qt_serial_get_received(void **userdata, size_t *available)
+static dc_status_t qt_serial_get_available(dc_custom_io_t *io, size_t *available)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
+
 #if defined(Q_OS_WIN)
 	if (device == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -366,9 +578,9 @@ static int qt_serial_get_transmitted(qt_serial_t *device)
 #endif
 }
 
-static dc_status_t qt_serial_set_timeout(void **userdata, long timeout)
+static dc_status_t qt_serial_set_timeout(dc_custom_io_t *io, long timeout)
 {
-	qt_serial_t *device = (qt_serial_t*) *userdata;
+	qt_serial_t *device = (qt_serial_t*) io->userdata;
 
 	if (device == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -378,26 +590,35 @@ static dc_status_t qt_serial_set_timeout(void **userdata, long timeout)
 	return DC_STATUS_SUCCESS;
 }
 
-dc_custom_serial_t qt_serial_ops = {
+dc_custom_io_t qt_serial_ops = {
 	.userdata = NULL,
-	.open = qt_serial_open,
-	.close = qt_serial_close,
-	.read = qt_serial_read,
-	.write = qt_serial_write,
-	.purge = qt_serial_flush,
-	.get_available = qt_serial_get_received,
-	.set_timeout = qt_serial_set_timeout,
+	.user_device = NULL,
+	.serial_open = qt_serial_open,
+	.serial_close = qt_serial_close,
+	.serial_read = qt_serial_read,
+	.serial_write = qt_serial_write,
+	.serial_purge = qt_serial_purge,
+	.serial_get_available = qt_serial_get_available,
+	.serial_set_timeout = qt_serial_set_timeout,
 // These doesn't make sense over bluetooth
 // NULL means NOP
-	.configure = NULL,
-	.set_dtr = NULL,
-	.set_rts = NULL,
-	.set_halfduplex = NULL,
-	.set_break = NULL
+	.serial_configure = NULL,
+	.serial_set_dtr = NULL,
+	.serial_set_rts = NULL,
+	.serial_set_halfduplex = NULL,
+	.serial_set_break = NULL,
+
+#ifdef BLE_SUPPORT
+	.packet_size  = 20,
+	.packet_open  = qt_ble_open,
+	.packet_close = qt_ble_close,
+	.packet_read  = qt_ble_read,
+	.packet_write = qt_ble_write,
+#endif
 };
 
-dc_custom_serial_t* get_qt_serial_ops() {
-	return (dc_custom_serial_t*) &qt_serial_ops;
+dc_custom_io_t* get_qt_serial_ops() {
+	return (dc_custom_io_t*) &qt_serial_ops;
 }
 
 }
