@@ -24,8 +24,6 @@ namespace DownloadFromDcGlobal {
 DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f),
 	downloading(false),
 	previousLast(0),
-	vendorModel(0),
-	productModel(0),
 	timer(new QTimer(this)),
 	dumpWarningShown(false),
 	ostcFirmwareCheck(0),
@@ -33,7 +31,7 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 {
 	diveImportedModel = new DiveImportedModel(this);
 	diveImportedModel->setDiveTable(&downloadTable);
-	vendorModel = new QStringListModel(vendorList);
+	vendorModel.setStringList(vendorList);
 	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
 	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
 
@@ -54,7 +52,8 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	ui.chooseLogFile->setEnabled(ui.logToFile->isChecked());
 	ui.selectAllButton->setEnabled(false);
 	ui.unselectAllButton->setEnabled(false);
-	ui.vendor->setModel(vendorModel);
+	ui.vendor->setModel(&vendorModel);
+	ui.product->setModel(&productModel);
 
 	progress_bar_text = "";
 
@@ -81,8 +80,7 @@ DownloadFromDCWidget::DownloadFromDCWidget(QWidget *parent, Qt::WindowFlags f) :
 	auto dc = SettingsObjectWrapper::instance()->dive_computer_settings;
 	if (!dc->dc_vendor().isEmpty()) {
 		ui.vendor->setCurrentIndex(ui.vendor->findText(dc->dc_vendor()));
-		productModel = new QStringListModel(productList[dc->dc_vendor()]);
-		ui.product->setModel(productModel);
+		productModel.setStringList(productList[dc->dc_vendor()]);
 		if (!dc->dc_product().isEmpty())
 			ui.product->setCurrentIndex(ui.product->findText(dc->dc_product()));
 	}
@@ -241,35 +239,17 @@ void DownloadFromDCWidget::updateState(states state)
 void DownloadFromDCWidget::on_vendor_currentIndexChanged(const QString &vendor)
 {
 	int dcType = DC_TYPE_SERIAL;
-	QAbstractItemModel *currentModel = ui.product->model();
-	if (!currentModel)
-		return;
-
-	productModel = new QStringListModel(productList[vendor]);
-	ui.product->setModel(productModel);
+	productModel.setStringList(productList[vendor]);
+	ui.product->setCurrentIndex(0);
 
 	if (vendor == QString("Uemis"))
 		dcType = DC_TYPE_UEMIS;
 	fill_device_list(dcType);
-
-	// Memleak - but deleting gives me a crash.
-	//currentModel->deleteLater();
 }
 
 void DownloadFromDCWidget::on_product_currentIndexChanged(const QString &product)
 {
-	// Set up the DC descriptor
-	dc_descriptor_t *descriptor = NULL;
-	descriptor = descriptorLookup[ui.vendor->currentText() + product];
-
-	// call dc_descriptor_get_transport to see if the dc_transport_t is DC_TRANSPORT_SERIAL
-	if (dc_descriptor_get_transport(descriptor) == DC_TRANSPORT_SERIAL) {
-		// if the dc_transport_t is DC_TRANSPORT_SERIAL, then enable the device node box.
-		ui.device->setEnabled(true);
-	} else {
-		// otherwise disable the device node box
-		ui.device->setEnabled(false);
-	}
+	updateDeviceEnabled();
 }
 
 void DownloadFromDCWidget::on_search_clicked()
@@ -378,7 +358,7 @@ void DownloadFromDCWidget::pickLogFile()
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.log");
 	QString logFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer download logfile"),
-					       filename, tr("Log files (*.log)"));
+					       filename, tr("Log files") + " (*.log)");
 	if (!logFile.isEmpty()) {
 		free(logfile_name);
 		logfile_name = copy_string(logFile.toUtf8().data());
@@ -404,7 +384,7 @@ void DownloadFromDCWidget::pickDumpFile()
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("subsurface.bin");
 	QString dumpFile = QFileDialog::getSaveFileName(this, tr("Choose file for dive computer binary dump file"),
-						filename, tr("Dump files (*.bin)"));
+						filename, tr("Dump files") + " (*.bin)");
 	if (!dumpFile.isEmpty()) {
 		free(dumpfile_name);
 		dumpfile_name = copy_string(dumpFile.toUtf8().data());
@@ -489,6 +469,22 @@ void DownloadFromDCWidget::on_ok_clicked()
 	accept();
 }
 
+void DownloadFromDCWidget::updateDeviceEnabled()
+{
+	// Set up the DC descriptor
+	dc_descriptor_t *descriptor = NULL;
+	descriptor = descriptorLookup.value(ui.vendor->currentText() + ui.product->currentText());
+
+	// call dc_descriptor_get_transport to see if the dc_transport_t is DC_TRANSPORT_SERIAL
+	if (dc_descriptor_get_transport(descriptor) == DC_TRANSPORT_SERIAL) {
+		// if the dc_transport_t is DC_TRANSPORT_SERIAL, then enable the device node box.
+		ui.device->setEnabled(true);
+	} else {
+		// otherwise disable the device node box
+		ui.device->setEnabled(false);
+	}
+}
+
 void DownloadFromDCWidget::markChildrenAsDisabled()
 {
 	ui.device->setEnabled(false);
@@ -511,7 +507,7 @@ void DownloadFromDCWidget::markChildrenAsDisabled()
 
 void DownloadFromDCWidget::markChildrenAsEnabled()
 {
-	ui.device->setEnabled(true);
+	updateDeviceEnabled();
 	ui.vendor->setEnabled(true);
 	ui.product->setEnabled(true);
 	ui.forceDownload->setEnabled(true);
@@ -550,7 +546,7 @@ void DownloadFromDCWidget::bluetoothSelectionDialogIsFinished(int result)
 		/* Make the selected Bluetooth device default */
 		QString selectedDeviceName = btDeviceSelectionDialog->getSelectedDeviceName();
 
-		if (selectedDeviceName == NULL || selectedDeviceName.isEmpty()) {
+		if (selectedDeviceName.isEmpty()) {
 			ui.device->setCurrentText(btDeviceSelectionDialog->getSelectedDeviceAddress());
 		} else {
 			ui.device->setCurrentText(selectedDeviceName);
