@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0
 #ifndef DIVETRIPMODEL_H
 #define DIVETRIPMODEL_H
 
@@ -23,6 +24,7 @@ public:
 		OTU,
 		MAXCNS,
 		PHOTOS,
+		COUNTRY,
 		LOCATION,
 		COLUMNS
 	};
@@ -36,8 +38,11 @@ public:
 	QString displayDepth() const;
 	QString displayDepthWithUnit() const;
 	QString displayTemperature() const;
+	QString displayTemperatureWithUnit() const;
 	QString displayWeight() const;
+	QString displayWeightWithUnit() const;
 	QString displaySac() const;
+	QString displaySacWithUnit() const;
 	int countPhotos(dive *dive) const;
 	int weight() const;
 	QString icon_names[4];
@@ -68,6 +73,7 @@ public:
 		OTU,
 		MAXCNS,
 		PHOTOS,
+		COUNTRY,
 		LOCATION,
 		COLUMNS
 	};
@@ -91,10 +97,13 @@ public:
 	DiveTripModel(QObject *parent = 0);
 	Layout layout() const;
 	void setLayout(Layout layout);
+	int columnWidth(int column);
+	void setColumnWidth(int column, int width);
 
 private:
 	void setupModelData();
 	QMap<dive_trip_t *, TripItem *> trips;
+	QVector<int> columnWidthMap;
 	Layout currentLayout;
 };
 
