@@ -55,6 +55,10 @@ while [[ $# -gt 0 ]] ; do
 			BUILD_MOBILE="1"
 			BUILD_DESKTOP="1"
 			;;
+		-create-appdir)
+			# we are building an AppImage as by product
+			CREATE_APPDIR="1"
+			;;
 		*)
 			echo "Unknown command line argument $arg"
 			;;
@@ -419,7 +423,7 @@ if [ "$BUILDGRANTLEE" = "1" ] ; then
 	cd build
 	cmake $OLDER_MAC_CMAKE -DCMAKE_BUILD_TYPE=Release \
 		-DCMAKE_INSTALL_PREFIX=$INSTALL_ROOT \
-		-DBUILD__TESTS=NO \
+		-DBUILD_TESTS=NO \
 		$SRC/grantlee
 	make -j4
 	make install
@@ -475,6 +479,7 @@ for (( i=0 ; i < ${#BUILDS[@]} ; i++ )) ; do
 		-DLIBDIVECOMPUTER_LIBRARIES=$INSTALL_ROOT/lib/libdivecomputer.a \
 		-DCMAKE_PREFIX_PATH=$CMAKE_PREFIX_PATH \
 		-DBTSUPPORT=${BTSUPPORT} \
+		-DCMAKE_INSTALL_PREFIX=${INSTALL_ROOT} \
 		$PRINTING $EXTRA_OPTS
 
 	if [ $PLATFORM = Darwin ] ; then
@@ -483,8 +488,16 @@ for (( i=0 ; i < ${#BUILDS[@]} ; i++ )) ; do
 	fi
 
 	LIBRARY_PATH=$INSTALL_ROOT/lib make -j4
+	LIBRARY_PATH=$INSTALL_ROOT/lib make install
 
-	if [ $PLATFORM = Darwin ] ; then
-		LIBRARY_PATH=$INSTALL_ROOT/lib make install
+	if [ "$CREATE_APPDIR" = "1" ] ; then
+		# if we create an AppImage this makes gives us a sane starting point
+		cd $SRC
+		mkdir -p ./appdir
+		mkdir -p appdir/usr/share/metainfo
+		mkdir -p appdir/usr/share/icons/hicolor/256x256/apps
+		cp -r ./install-root/* ./appdir/usr
+		cp subsurface/appdata/subsurface.appdata.xml appdir/usr/share/metainfo/
+		cp subsurface/icons/subsurface-icon.png appdir/usr/share/icons/hicolor/256x256/apps/
 	fi
 done
