@@ -1,8 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0
 #include "testparse.h"
 #include "core/dive.h"
 #include "core/file.h"
 #include "core/divelist.h"
 #include <QTextStream>
+#include "core/qthelper.h"
 
 /* We have to use a macro since QCOMPARE
  * can only be called from a test method
@@ -41,15 +43,6 @@ void TestParse::cleanup()
 	sqlite3_close(_sqlite3_handle);
 }
 
-char *intdup(int index)
-{
-	char tmpbuf[21];
-
-	snprintf(tmpbuf, sizeof(tmpbuf) - 2, "%d", index);
-	tmpbuf[20] = 0;
-	return strdup(tmpbuf);
-}
-
 int TestParse::parseCSV(int units, std::string file)
 {
 	// some basic file parsing tests
@@ -59,7 +52,7 @@ int TestParse::parseCSV(int units, std::string file)
 	char *params[55];
 	int pnr = 0;
 
-	params[pnr++] = strdup(strdup("numberField"));
+	params[pnr++] = strdup("numberField");
 	params[pnr++] = intdup(0);
 	params[pnr++] = strdup("dateField");
 	params[pnr++] = intdup(1);
@@ -145,7 +138,7 @@ void TestParse::testParse()
 {
 	QCOMPARE(parseCSV(0, SUBSURFACE_TEST_DATA "/dives/test41.csv"), 0);
 	fprintf(stderr, "number of dives %d \n", dive_table.nr);
-	
+
 	QCOMPARE(parseDivingLog(), 0);
 	fprintf(stderr, "number of dives %d \n", dive_table.nr);
 
@@ -164,7 +157,7 @@ void TestParse::testParseDM4()
 {
 	QCOMPARE(sqlite3_open(SUBSURFACE_TEST_DATA "/dives/TestDiveDM4.db", &_sqlite3_handle), 0);
 	QCOMPARE(parse_dm4_buffer(_sqlite3_handle, 0, 0, 0, &dive_table), 0);
-	
+
 	QCOMPARE(save_dives("./testdm4out.ssrf"), 0);
 	FILE_COMPARE("./testdm4out.ssrf",
 		SUBSURFACE_TEST_DATA "/dives/TestDiveDM4.xml");
@@ -231,10 +224,8 @@ void TestParse::testParseHUDC()
 
 void TestParse::testParseNewFormat()
 {
-	struct dive *dive;
 	QDir dir;
 	QStringList filter;
-	QString firstLine;
 	QStringList files;
 
 	/*
@@ -251,116 +242,10 @@ void TestParse::testParseNewFormat()
 	 */
 
 	for (int i = 0; i < files.size(); ++i) {
-		QString delta;
-		QStringList currColumns;
-		QStringList headers;
-		QString file = QString::fromLatin1(SUBSURFACE_TEST_DATA "/dives/").append(files.at(i));
-		QFile f(file);
-
-		/*
-		 * Parse the sample interval if available from CSV
-		 * header.
-		 */
-
-		f.open(QFile::ReadOnly);
-		while ((firstLine = f.readLine().trimmed()).length() > 0 && !f.atEnd()) {
-			if (firstLine.contains("//Log interval: "))
-				delta = firstLine.remove(QString::fromLatin1("//Log interval: ")).trimmed().remove(QString::fromLatin1(" s"));
-		}
-		firstLine = f.readLine().trimmed();
-
-		/*
-		 * Parse the field configuration from the CSV header.
-		 */
-
-		currColumns = firstLine.split(';');
-		Q_FOREACH (QString columnText, currColumns) {
-			if (columnText == "Time") {
-				headers.append("Sample time");
-			} else if (columnText == "Depth") {
-				headers.append("Sample depth");
-			} else if (columnText == "Temperature") {
-				headers.append("Sample temperature");
-			} else if (columnText == "NDT") {
-				headers.append("Sample NDL");
-			} else if (columnText == "TTS") {
-				headers.append("Sample TTS");
-			} else if (columnText == "pO2_1") {
-				headers.append("Sample sensor1 pO₂");
-			} else if (columnText == "pO2_2") {
-				headers.append("Sample sensor2 pO₂");
-			} else if (columnText == "pO2_3") {
-				headers.append("Sample sensor3 pO₂");
-			} else if (columnText == "Ceiling") {
-				headers.append("Sample ceiling");
-			} else if (columnText == "Tank pressure") {
-				headers.append("Sample pressure");
-			} else {
-				// We do not know about this value
-				qDebug() << "Seabear import found an un-handled field: " << columnText;
-				headers.append("");
-			}
-			f.close();
-		}
-
-		/*
-		 * Validate the parsing routine returns success.
-		 */
-
-		char *params[42];
-		int pnr = 0;
-
-		params[pnr++] = strdup("timeField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample time")));
-		params[pnr++] = strdup("depthField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample depth")));
-		params[pnr++] = strdup("tempField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample temperature")));
-		params[pnr++] = strdup("po2Field");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample pO₂")));
-		params[pnr++] = strdup("o2sensor1Field");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample sensor1 pO₂")));
-		params[pnr++] = strdup("o2sensor2Field");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample sensor2 pO₂")));
-		params[pnr++] = strdup("o2sensor3Field");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample sensor3 pO₂")));
-		params[pnr++] = strdup("cnsField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample CNS")));
-		params[pnr++] = strdup("ndlField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample NDL")));
-		params[pnr++] = strdup("ttsField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample TTS")));
-		params[pnr++] = strdup("stopdepthField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample stopdepth")));
-		params[pnr++] = strdup("pressureField");
-		params[pnr++] = intdup(headers.indexOf(tr("Sample pressure")));
-		params[pnr++] = strdup("setpointField");
-		params[pnr++] = intdup(-1);
-		params[pnr++] = strdup("numberField");
-		params[pnr++] = intdup(-1);
-		params[pnr++] = strdup("separatorIndex");
-		params[pnr++] = intdup(2);
-		params[pnr++] = strdup("units");
-		params[pnr++] = intdup(0);
-		params[pnr++] = strdup("delta");
-		params[pnr++] = strdup(delta.toUtf8().data());
-		params[pnr++] = NULL;
-
-		QCOMPARE(parse_seabear_csv_file(file.toUtf8().data(),
-					params, pnr - 1, "csv"), 0);
 
+		QCOMPARE(parse_seabear_log(QString::fromLatin1(SUBSURFACE_TEST_DATA
+			"/dives/").append(files.at(i)).toLatin1().data()), 0);
 		QCOMPARE(dive_table.nr, i + 1);
-
-		/*
-		 * Set artificial but static dive times so the result
-		 * can be compared to saved one.
-		 */
-
-		if (dive_table.nr > 0) {
-			dive = dive_table.dives[dive_table.nr - 1];
-			dive->when = 1255152761 + 7200 * i;
-			dive->dc.when = 1255152761 + 7200 * i;
-		}
 	}
 
 	fprintf(stderr, "number of dives %d \n", dive_table.nr);
@@ -410,7 +295,7 @@ int TestParse::parseCSVmanual(int units, std::string file)
 	char *params[55];
 	int pnr = 0;
 
-	params[pnr++] = strdup(strdup("numberField"));
+	params[pnr++] = strdup("numberField");
 	params[pnr++] = intdup(0);
 	params[pnr++] = strdup("dateField");
 	params[pnr++] = intdup(1);
@@ -483,9 +368,75 @@ void TestParse::exportCSVDiveDetails()
 	clear_dive_file_data();
 }
 
+int TestParse::parseCSVprofile(int units, std::string file)
+{
+	verbose = 1;
+	char *params[55];
+	int pnr = 0;
+
+	params[pnr++] = strdup("dateField");
+	params[pnr++] = intdup(1);
+	params[pnr++] = strdup("datefmt");
+	params[pnr++] = intdup(2);
+	params[pnr++] = strdup("starttimeField");
+	params[pnr++] = intdup(2);
+	params[pnr++] = strdup("numberField");
+	params[pnr++] = intdup(0);
+	params[pnr++] = strdup("timeField");
+	params[pnr++] = intdup(3);
+	params[pnr++] = strdup("depthField");
+	params[pnr++] = intdup(4);
+	params[pnr++] = strdup("tempField");
+	params[pnr++] = intdup(5);
+	params[pnr++] = strdup("pressureField");
+	params[pnr++] = intdup(6);
+	params[pnr++] = strdup("units");
+	params[pnr++] = intdup(units);
+	params[pnr++] = NULL;
+
+	return parse_csv_file(file.c_str(), params, pnr - 1, "csv");
+}
+
+void TestParse::exportCSVDiveProfile()
+{
+	parse_file(SUBSURFACE_TEST_DATA "/dives/test40.xml");
+
+	export_dives_xslt("testcsvexportprofile.csv", 0, 0, "xml2csv.xslt");
+	export_dives_xslt("testcsvexportprofileimperial.csv", 0, 1, "xml2csv.xslt");
+
+	clear_dive_file_data();
+
+	parseCSVprofile(1, "testcsvexportprofileimperial.csv");
+	export_dives_xslt("testcsvexportprofile2.csv", 0, 0, "xml2csv.xslt");
+
+	FILE_COMPARE("testcsvexportprofile2.csv",
+		"testcsvexportprofile.csv");
+
+	clear_dive_file_data();
+}
+
+void TestParse::exportUDDF()
+{
+	parse_file(SUBSURFACE_TEST_DATA "/dives/test40.xml");
+
+	export_dives_xslt("testuddfexport.uddf", 0, 1, "uddf-export.xslt");
+
+	clear_dive_file_data();
+
+	parse_file("testuddfexport.uddf");
+	export_dives_xslt("testuddfexport2.uddf", 0, 1, "uddf-export.xslt");
+
+	FILE_COMPARE("testuddfexport.uddf",
+		"testuddfexport2.uddf");
+
+	clear_dive_file_data();
+}
+
 void TestParse::testExport()
 {
 	exportCSVDiveDetails();
+	exportCSVDiveProfile();
+	exportUDDF();
 }
 
 
